from setuptools import setup, Extension

extensions = [Extension("libiio",
                        ["iio/iio.c"],
                        include_dirs=['iio'],
                        libraries=['png', 'jpeg', 'tiff'],
                        depends=["iio/iio.h"])]


# from https://stackoverflow.com/a/38525461
from distutils.command.install_lib import install_lib as _install_lib
import os
import re
def batch_rename(src, dst, src_dir_fd=None, dst_dir_fd=None):
    '''Same as os.rename, but returns the renaming result.'''
    os.rename(src, dst,
              src_dir_fd=src_dir_fd,
              dst_dir_fd=dst_dir_fd)
    return dst

class _CommandInstall(_install_lib):
    def __init__(self, *args, **kwargs):
        _install_lib.__init__(self, *args, **kwargs)

    def install(self):
        # let the distutils' install_lib do the hard work
        outfiles = _install_lib.install(self)
        # batch rename the outfiles:
        # for each file, match string between
        # second last and last dot and trim it
        matcher = re.compile('\.([^.]+)\.so$')
        return [batch_rename(file, re.sub(matcher, '.so', file))
                for file in outfiles]

setup(name="iio",
      version='0.0.4a1',
      author="Jérémy Anger, Gabriele Facciolo",
      author_email="angerj.dev@gmail.com",
      description="Python wrapper to iio",
      url='https://github.com/mnhrdt/iio',
      classifiers=[
          "Operating System :: OS Independent",
      ],
      py_modules=['iio'],
      ext_modules=extensions,
      cmdclass={
          'install_lib': _CommandInstall,
      },
)




import os.path
import ctypes
from ctypes import *
from ctypes.util import find_library
import numpy as np

libiio = CDLL(os.path.join(os.path.abspath(os.path.dirname(__file__)), 'libiio.so'))

libc = ctypes.CDLL(find_library('c'))
libc.free.argtypes = [ctypes.c_void_p]
libc.free.restype = ctypes.c_void_p

iioread = libiio.iio_read_image_float_vec
iioread.argtypes = [c_char_p, POINTER(c_int), POINTER(c_int), POINTER(c_int)]
iioread.restype = POINTER(c_float)

iiosave = libiio.iio_write_image_float_vec
iiosave.argtypes = [c_char_p, np.ctypeslib.ndpointer(c_float),c_int,c_int,c_int]
iiosave.restype = None

def read(filename):
   w = c_int()
   h = c_int()
   nch = c_int()

   ptr = iioread(filename.encode('utf-8'), w, h, nch)
   data = np.ctypeslib.as_array(ptr, (h.value, w.value, nch.value)).copy()
   libc.free(ptr)
   return data

def write(filename, data):
   h = data.shape[0]
   w = len(data.shape) <= 1 and 1 or data.shape[1]
   nch = len(data.shape) <= 2 and 1 or data.shape[2]

   data = np.ascontiguousarray(data, dtype='float32')
   iiosave(filename.encode('utf-8'), data, w, h, nch)

__all__ = {
    'read': read,
    'write': write,
}

